﻿/******************************************************************************
  * 文件名称: JsonHelper.cs
 * 文件说明: Json业务的封装，处理JSON串的封装，解析等
 * 当前版本: V1.0
 * 创建日期: 2022-04-14
 *
 * 2020-04-14: 增加 setIrisInpar 方法
 * 2020-04-14: 增加 setIrisInpar（重载） 方法
 * 2020-04-14: 增加 getIrisExceptionJson 方法
 * 2020-04-14: 增加 parseBusinessJson 方法

 ******************************************************************************/
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using PTMedicalInsurance.Common;
using System.IO;
using PTMedicalInsurance.Variables;

namespace PTMedicalInsurance.Helper
{
    class JsonHelper
    {
       

        /// <summary>
        /// 根据JSonPath查找节点值，如果节点不存在则返回空值
        /// </summary>
        /// <param name="jo"></param>
        /// <param name="jsonPath"></param>
        /// <returns></returns>
        public static string getDestValue(JObject jo, string destPath)
        {
            JToken jt = jo.SelectToken("$." + destPath);
            if (jt != null)
            {
                return jt.ToString();
            }
            else
            {
                Global.writeLog(destPath + "的JToken属性值为空");
                return "";
            }
        }


        public static string getDestProperty(JObject jo, string propertyName)
        {
            JProperty jp = jo.Property(propertyName);
            if (jp != null)
            {
                string jpStr = jp.ToString();
                string rtnResult = "{ " + jpStr + "}";
                return rtnResult;
            }
            else
            {
                Global.writeLog(propertyName + "的JProperty属性值为空");
                return "";
            }
            
        }
        /// <summary>
        /// 压缩JSON，占用体积减小(重载)
        /// </summary>
        /// <param name="json"></param>
        /// <returns></returns>
        public static string Compress(string json)
  {
      StringBuilder sb = new StringBuilder();
      using (StringReader reader = new StringReader(json))
      {
          int ch = -1;
          int lastch = -1;
         bool isQuoteStart = false;
          while ((ch = reader.Read()) > -1)
         {
             if ((char) lastch != '\\' && (char) ch == '\"')
             {
                 if (!isQuoteStart)
                 {
                     isQuoteStart = true;
                 }
                 else
                 {
                     isQuoteStart = false;
                 }
             }
             if (!Char.IsWhiteSpace((char) ch) || isQuoteStart)
             {
                 sb.Append((char) ch);
             }
             lastch = ch;
         }
     }
     return sb.ToString();
 }
        public static string Compress(JObject jsonObj)
        {
            //https://blog.csdn.net/yangjiaosun/article/details/103717256
            return Newtonsoft.Json.JsonConvert.SerializeObject(jsonObj);
        }

        /// <summary>
        /// 组织异常JSON串
        /// </summary>
        /// <param name="errorCode"></param>
        /// <param name="errorMessage"></param>
        /// <returns></returns>
        public static JObject setExceptionJson(int errorCode, string eventName, string errorMessage)
        {
            dynamic joRtn = new JObject();
            joRtn.errorCode = errorCode;
            joRtn.errorMessage = eventName + "提示：" + errorMessage;
            return joRtn;
        }

        /// <summary>
        /// 组织中心入参(明文)。部分代码是需要解析明文入参的，为了方便增加该方法
        /// </summary>
        /// <param name="infno"></param>
        /// <param name="input"></param>
        /// <returns></returns>
        public static string setCenterInpar_plain(string infno, string input)
        {
            dynamic Jo = new JObject();
            Jo.infno = infno;
            Global.curEvt.msgid = Global.inf.hospitalNO + DateTime.Now.ToString("yyyyMMddHHmmssffff");
            Jo.msgid = Global.curEvt.msgid;

            if (Global.inf.hospitalDr.ToString() == "18")
            {
                Jo.mdtrtarea_admvs = "420104";
                Global.pat.mdtrtarea_admvs = "420104";
            }
            else if (Global.inf.hospitalDr.ToString() == "")
            {
                Jo.mdtrtarea_admvs = "420104";
                Global.pat.mdtrtarea_admvs = "420104";
            }
            else
            {
                Jo.mdtrtarea_admvs = "420100";
                Global.pat.mdtrtarea_admvs = "420100";
            }

            //Jo.mdtrtarea_admvs = Global.inf.areaCode;
            Jo.insuplc_admdvs = Global.pat.insuplc_admdvs;
            Jo.recer_sys_code = Global.inf.recivedSystem;
            Jo.dev_safe_info = Global.inf.dev_safe_info; //设备编号
            Jo.cainfo = Global.inf.cainfo;//设备安全信息
            if (Global.inf.hospitalDr.ToString() == "18")
            {
                Jo.cainfo = "1d960e8bbe077a482cc631af7c054008";//武汉普瑞硚口区医保密钥20230918
            }
            Jo.signtype = Global.inf.signatureType; ;
            Jo.infver = Global.inf.version; ;
            Jo.opter_type = Global.user.type; ;
            Jo.opter = Global.user.ID; ;
            Jo.opter_name = Global.user.name; ;
            Jo.inf_time = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            Jo.fixmedins_code = Global.inf.hospitalNO;
            Jo.fixmedins_name = Global.inf.hospitalName;
            Jo.sign_no = Global.curEvt.signno;
            JObject joInput = (JObject)Newtonsoft.Json.JsonConvert.DeserializeObject(input);
            Jo.Add("input", JObject.FromObject(joInput));
            return Jo.ToString();
        }

        /// <summary>
        /// 组织中心入参(明文)。部分代码是需要解析明文入参的，为了方便增加该方法
        /// </summary>
        /// <param name="infno"></param>
        /// <param name="input"></param>
        /// <returns></returns>
        public static string setCenterInpar_plain(string infno, JObject joInput)
        {
            dynamic Jo = new JObject();
            Jo.infno = infno;
            Global.curEvt.msgid = Global.inf.hospitalNO + DateTime.Now.ToString("yyyyMMddHHmmssffff");
            Jo.msgid = Global.curEvt.msgid;

            if (Global.inf.hospitalDr.ToString() == "18")  //武汉老院
            {
                Jo.mdtrtarea_admvs = "420104";
                Global.pat.mdtrtarea_admvs = "420104";
            }
            else if (Global.inf.hospitalDr.ToString() == "") //武汉新院医疗机构信息未下发
            {
                Jo.mdtrtarea_admvs = "420104";
                Global.pat.mdtrtarea_admvs = "420104";
            }
            else
            {
                Jo.mdtrtarea_admvs = "420100";
                Global.pat.mdtrtarea_admvs = "420100";
            }

            //Jo.mdtrtarea_admvs = Global.inf.areaCode;
            Jo.insuplc_admdvs = Global.pat.insuplc_admdvs;
            Jo.recer_sys_code = Global.inf.recivedSystem;
            Jo.dev_no = Global.inf.dev_no;
            Jo.dev_safe_info = Global.inf.dev_safe_info; //设备编号
            Jo.cainfo = Global.inf.cainfo;//设备安全信息
            if (Global.inf.hospitalDr.ToString() == "18")
            {
                Jo.cainfo = "1d960e8bbe077a482cc631af7c054008";//武汉普瑞硚口区医保密钥20230918
            }
            Jo.signtype = Global.inf.signatureType; 
            Jo.infver = Global.inf.version; 
            Jo.opter_type = Global.user.type; 
            Jo.opter = Global.user.ID; 
            Jo.opter_name = Global.user.name; 
            Jo.inf_time = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            Jo.fixmedins_code = Global.inf.hospitalNO;
            Jo.fixmedins_name = Global.inf.hospitalName;
            Jo.sign_no = Global.curEvt.signno;
            Jo.Add("input", joInput);
            return Jo.ToString();
        }

        /// <summary>
        /// 组织中心入参
        /// </summary>
        /// <param name="infno"></param>
        /// <param name="input"></param>
        /// <returns></returns>
        public static string setCenterInpar(string infno, string input)
        {
            dynamic Jo = new JObject();
            Jo.infno = infno;
            Global.curEvt.msgid = Global.inf.hospitalNO + DateTime.Now.ToString("yyyyMMddHHmmssffff");
            Jo.msgid = Global.curEvt.msgid;

            if (Global.inf.areaCode == "")
            {
                Global.inf.areaCode = "420104";
            }
 
            if (Global.pat.insuplc_admdvs == "")
            {
                Global.pat.insuplc_admdvs = Global.inf.areaCode;
            }

            Jo.insuplc_admdvs = Global.pat.insuplc_admdvs;
            


            if (Global.inf.hospitalDr.ToString() == "18") ////武汉老院
            {
                Jo.mdtrtarea_admvs = "420104";
                Global.pat.mdtrtarea_admvs = "420104";
            }
            else if (Global.inf.hospitalDr.ToString() == "") ////武汉新院测试
            {
                Jo.mdtrtarea_admvs = "420104";
                Global.pat.mdtrtarea_admvs = "420104";
            }
            else
            {
                Jo.mdtrtarea_admvs = "420104";
                Global.pat.mdtrtarea_admvs = "420104";
            }

            Jo.recer_sys_code = "1";
            Jo.dev_no = Global.curEvt.mac;
            Jo.dev_safe_info = Global.inf.dev_safe_info; //设备编号
            Jo.cainfo = Global.inf.cainfo ;//
            if (Global.inf.hospitalDr.ToString() == "18")
            {
                Jo.cainfo = "1d960e8bbe077a482cc631af7c054008";//武汉普瑞硚口区医保密钥20230918
            }
            Jo.signtype = "SM3"; ;
            Jo.infver = "V2.0";
            //Jo.opter_type = Global.user.type; ;
            Jo.opter_type = "1";//1经办人  2自助终端  3移动终端
            Jo.opter = Global.user.ID; ;
            Jo.opter_name = Global.user.name; ;
            Jo.inf_time = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            Jo.fixmedins_code = Global.inf.hospitalNO;
            Jo.fixmedins_name = Global.inf.hospitalName;
            Jo.sign_no = Global.curEvt.signno;

            if (input != "")
            {
                if (infno == "1608") 
                {
                    Jo.input = input;

                } else
                {
                    JObject joInput = (JObject)Newtonsoft.Json.JsonConvert.DeserializeObject(input);

                    //Jo.cainfo = "";
                    Jo.Add("input", JObject.FromObject(joInput));

                }
                

            }
            else
            {
                //Jo.cainfo = "";
                Jo.Add("input", "");
            }

            return Jo.ToString();
        }

        /// <summary>
        /// 组织中心入参
        /// </summary>
        /// <param name="infno"></param>
        /// <param name="input"></param>
        /// <returns></returns>
        public static string setCenterInpar(string infno, JObject joInput)
        {
            dynamic Jo = new JObject();
            Jo.infno = infno;
            Global.curEvt.msgid = Global.inf.hospitalNO + DateTime.Now.ToString("yyyyMMddHHmmssffff");
            Jo.msgid = Global.curEvt.msgid;
            Jo.mdtrtarea_admvs = Global.inf.areaCode;

            if (Global.pat.insuplc_admdvs == "")
            {
                Global.pat.insuplc_admdvs = Global.inf.areaCode;
            }
            Jo.insuplc_admdvs = Global.pat.insuplc_admdvs;

            if (Global.inf.hospitalDr.ToString() == "18")
            {
                Jo.mdtrtarea_admvs = "420104";
                Global.pat.mdtrtarea_admvs = "420104";
            }

            Jo.recer_sys_code = "HuBei";
            Jo.dev_no = Global.curEvt.mac; 
            Jo.dev_safe_info = Global.inf.dev_safe_info; //设备编号
            Jo.cainfo = Global.inf.cainfo;//
            if (Global.inf.hospitalDr.ToString() == "18")
            {
                Jo.cainfo = "1d960e8bbe077a482cc631af7c054008";//武汉普瑞硚口区医保密钥20230918
            }
            Jo.signtype = "asn1"; ;
            Jo.infver = "v1.0";
            Jo.opter_type = Global.user.type; ;
            Jo.opter = Global.user.ID; 
            Jo.opter_name = Global.user.name; ;
            Jo.inf_time = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            Jo.fixmedins_code = Global.inf.hospitalNO;
            Jo.fixmedins_name = Global.inf.hospitalName;
            Jo.sign_no = Global.curEvt.signno;

            string nonce = Guid.NewGuid().ToString();         //非重复的随机字符串（十分钟内不能重复）
            string timestamp = TimeStamp.get13().ToString();  //当前时间戳（秒）
            string BusinessID = Global.inf.BusinessID;        //服务商ID
            string InsuHosID = Global.inf.hospitalNO;         //医疗机构ID
            string CreditID = Global.inf.CreditID;            //服务商统一社会信用代码                

            //昆明增加头部信息
            string sTemp = timestamp + BusinessID + nonce;
            //Sha256 加密生成的签名 sha256(hsf_timestamp + infosyssign + hsf_nonce)
           // string cainfo = "1d960e8bbe077a482cc631af7c054008";//Encrypt.SHA256EncryptStr(sTemp);最新医保密钥20230918

           // Jo.cainfo = cainfo;
            Jo.Add("input", JObject.FromObject(joInput));

            return Jo.ToString();
        }

        /// <summary>
        /// 组织Iris入参
        /// </summary>
        /// <param name="code"></param>
        /// <param name="joParam"></param>
        /// <returns></returns>
        public static JObject setIrisInpar(string code, JObject joParam)
        {
            try
            {
                dynamic joInparam = new JObject();
                joInparam.code = code;
                dynamic joTmp = new JObject();
                JArray jaParam = new JArray();
                jaParam.Add(joParam);
                joInparam.Add("params", JArray.FromObject(jaParam));
                joInparam.Add("session", Global.curEvt.jaSession);
                return joInparam;
            }
            catch (Exception ex)
            {
                return setExceptionJson(-1, "setIrisInpar:", ex.Message);
            }
        }

        /// <summary>
        /// 组织Iris入参
        /// </summary>
        /// <param name="code"></param>
        /// <param name="joParam"></param>
        /// <returns></returns>
        public static JObject setIrisInpar(string code, JArray jaParams)
        {
            try
            {   
                dynamic joInparam = new JObject();
                joInparam.code = code;
                dynamic joTmp = new JObject();
                joInparam.Add("params", jaParams);
                joInparam.Add("session", Global.curEvt.jaSession);
                return joInparam;
            }
            catch (Exception ex)
            {
                return setExceptionJson(-1, "setIrisInpar:", ex.Message);
            }

        }

        /// <summary>
        /// 解析中心返参
        /// </summary>
        /// <param name="joRtn"></param>
        /// <param name="errorMsg"></param>
        /// <returns></returns>
        public static int parseCenterRtnValue(JObject joRtn,  out string errorMsg)
        {
            try
            {
                string errcode = getDestValue(joRtn, "errcode");
                string errorCode = getDestValue(joRtn, "errorCode");
                if (errcode != "")
                {
                    errorMsg = getDestValue(joRtn, "errmsg");
                    return -1;
                }
                else if (errorCode != "")
                {
                    errorMsg = getDestValue(joRtn, "errorMessage");
                    return -2;
                }
                else
                {
                    errorMsg = getDestValue(joRtn, "err_msg");
                    return int.Parse(getDestValue(joRtn, "infcode"));
                }
            }
            catch (Exception ex)
            {
                errorMsg = "解析中心返参发生异常：" + ex.Message;
                return -1;
            }
        }

        /// <summary>
        /// 组织IRIS返参
        /// </summary>
        /// <param name="errorCode"></param>
        /// <param name="errorMessage"></param>
        /// <param name="joResult"></param>
        /// <returns></returns>
        public static JObject setIrisReturnValue(int errorCode, string errorMessage, JObject joResult)
        {
            try
            {
                dynamic joRtn = new JObject();
                joRtn.errorCode = errorCode;
                joRtn.errorMessage = errorMessage;
                joRtn.Add("result", joResult);
                return joRtn;
            }
            catch (Exception ex)
            {
                return setExceptionJson(-1, "setIrisReturnValue:", ex.Message);
            }
        }

        /// <summary>
        /// 解析IRIS返参
        /// </summary>
        /// <param name="joRtn"></param>
        /// <param name="errorMsg"></param>
        /// <returns></returns>
        public static int parseIrisRtnValue(JObject joRtn, out string errorMsg)
        {
            try
            {
                errorMsg = getDestValue(joRtn, "errorMessage");
                return int.Parse(getDestValue(joRtn, "errorCode"));
            }
            catch (Exception ex)
            {
                errorMsg = "解析Iris返参发生异常：" + ex.Message;
                return -1;
            }
        }

        /// <summary>
        /// 解析业务JSON串（业务串固定格式为{errorCode = ,errorMessage = ,result ={}}）
        /// </summary>
        /// <param name="json"></param>
        /// <param name="errorMsg"></param>
        /// <returns></returns>
        public static int parseCenterReturnJson(JObject json, out string errorMsg)
        {
            try
            {
                errorMsg = "";
                if (int.Parse(json["infcode"].ToString()) != 0)
                {
                    errorMsg = json["err_msg"].ToString();
                }
                return int.Parse(json["infcode"].ToString());
            }
            catch (Exception ex)
            {
                errorMsg = "parseCenterReturnJson" + ex.Message;
                return -1;
            }

        }
    }
}
